@@ -57,6 +57,6 @@ static inline opus_int32 le_int(opus_int32 i)
 
 void adjust_wav_mapping(int mapping_family, int channels, unsigned char *stream_map);
 
-int write_wav_header(FILE *file, int rate, int mapping_family, int channels);
+int write_wav_header(FILE *file, int rate, int mapping_family, int channels, int fp);
 
 #endif
